package org.dive4elements.river.exports.injector;

import org.apache.log4j.Logger;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.ArtifactNamespaceContext;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.artifacts.ContextInjector;
import org.dive4elements.artifacts.common.utils.XMLUtils;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.access.RangeAccess;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import static org.dive4elements.river.exports.injector.InjectorConstants.CURRENT_KM;


public class FixWQInjector
implements ContextInjector
{
    private static final Logger log = Logger.getLogger(FixWQInjector.class);

    public static final Double INVALID_KM = Double.valueOf(-1d);
    public static final String XPATH_CHART_CURRENTKM =
        "/art:action/art:attributes/art:currentKm/@art:km";
    @Override
    public void setup(Element cfg) {

    }

    @Override
    public void injectContext(
        CallContext ctx,
        Artifact artifact,
        Document request) {
        Double km = getCurrentKmFromRequest(request);
        if (km == Double.NaN) {
            RangeAccess access = new RangeAccess((D4EArtifact)artifact);
            km = access.getFrom();
            log.debug("from access");
        }
        log.debug("current km: " + km);
        ctx.putContextValue(CURRENT_KM, km);
    }

    private Double getCurrentKmFromRequest(Document request) {

        String km = XMLUtils.xpathString(
            request,
            XPATH_CHART_CURRENTKM,
            ArtifactNamespaceContext.INSTANCE);

        if (km == null) {
            return Double.NaN;
        }

        try {
            return Double.valueOf(km);
        }
        catch (NumberFormatException nfe) {
            return Double.NaN;
        }
    }
}
